
%% Creation of several types rectangular mask
% Ales Zita 2011

%% Description
% This piece of code creates the rectangular logical mask 
% for given dimension and angle

%% Code

function Mask = CreateMask(dim, type, angle)
%% Input arguments
if nargin > 3 || nargin < 2
    'Function takes 2 or 3 arguments'
    return;
end
if nargin < 3
    angle = 0;
end

%% Init
Mask = uint8(ones(dim, dim));

%% Mask creation
switch type
    
    %angled rectangle
    case 'angled'
        
        if angle < pi/4
            trans = true;
            angle = pi/2 - angle;
        else
            trans = false;
        end
        
        dx = sin(angle);
        dy = cos(angle);
        dim = dim/2;
        
        for i=1:dx * dim
            for j=1:dy*(dim-i+1)
                Mask(j,i) = 0;
            end
        end
        Mask = rot90(Mask);
        for i=1:dx * dim
            for j=1:dy*(dim-i+1)
                Mask(j,i) = 0;
            end
        end
        Mask = rot90(Mask);
        for i=1:dx * dim
            for j=1:dy*(dim-i+1)
                Mask(j,i) = 0;
            end
        end
        Mask = rot90(Mask);
        for i=1:dx * dim
            for j=1:dy*(dim-i+1)
                Mask(j,i) = 0;
            end
        end
        
        if trans
            Mask = Mask';
        end
        
    % Classic rectangular mask    
    case 'rectangle'
        Mask = uint8(ones(dim, dim));
        
    % 45 deg rectangular mask    
    case 'oblique'
        
        for i=1:dim/2
            for j=1:dim/2 - i
                Mask(j,i) = 0;
            end
        end
        Mask = rot90(Mask);
        for i=1:dim/2
            for j=1:dim/2 - i
                Mask(j,i) = 0;
            end
        end
        Mask = rot90(Mask);
        for i=1:dim/2
            for j=1:dim/2 - i
                Mask(j,i) = 0;
            end
        end
        Mask = rot90(Mask);
        for i=1:dim/2
            for j=1:dim/2 - i
                Mask(j,i) = 0;
            end
        end
        
    % Testing mask    
    case 'test'
        Mask = uint8([
            0 0 0 0 1 1 0 0 0 0 ;
            0 0 0 1 1 1 1 0 0 0 ;
            0 0 1 1 1 1 1 1 0 0 ;
            0 1 1 1 1 1 1 1 1 0 ;
            1 1 1 1 1 1 1 1 1 1 ;
            1 1 1 1 1 1 1 1 1 1 ;
            0 1 1 1 1 1 1 1 1 0 ;
            0 0 1 1 1 1 1 1 0 0 ;
            0 0 0 1 1 1 1 0 0 0 ;
            0 0 0 0 1 1 0 0 0 0 ]);
    otherwise
        'Undefined mask type'
end;
end

