
%% Masked Normalized Crosscorrelation
% Ales Zita 2011

%% Description
% This subroutine computes Masked Normalized Cross Correlation 
% Same as the normal NCC with addition of masking both function and kernel
% allowing to have unusual shapes of kernel

%% Code
% template = kernel
% image is the basic image function
% mask is selfexplanatory

function result = CrossCorrelation(template, image, mask)
    %% Initial args
    if size(template) ~= size(mask)
        'mask is different size'
        return;
    end;
    
    % Global vars
    result = zeros(size(image));
    half = floor(size(mask,1)/2);
    
    % Odd size kernel check
    t = template;
    if mod(size(mask,1),2) == 0
        mask = mask(1:size(mask,1)-1,1:size(mask,2)-1);
        t = t(1:size(t,1)-1,1:size(t,2)-1);
        t = t.*mask;
    end;
    
    % masked t mean 
    noel = sum(sum(mask));
    tm = sum(sum(t))/noel * mask;
    
    %% Main loop
    for n=half+1:size(image,2)-half
        for m=half+1:size(image,1)-half
            
            % Masked image
            f = image(m-half:m+half,n-half:n+half);
            f = f.*mask;
            
            % Masked f mean
            fm = sum(sum(f))/noel * mask;
            
            % Numerator
            cor = sum(sum((f - fm).*(t - tm)));
            
            % Denominator
            norm = sqrt( sum(sum((f-fm).*(f-fm))) * sum(sum((t-tm).*(t-tm))));
            
            % Result
            result(m,n) = cor/norm;
            
        end;
    end;
end