
%% Extremal point finding subroutine
% Ales Zita 2011

%% Description
% To find most extreme point in the direction of given angle
% on segmented image using simple DDA-like algotithm
% Typically starting in the middle of correlation kernel region

%% Code
% image - segmented image
% start - starting coords
% angle

function [x,y] = FindExtrema(image, startx, starty, angle)
    % False start detection
    if image(starty,startx) ~= 1
        %'False beginning'
        x = startx;
        y = starty;
        return;
    end
    
    %% initialization
    dx = sin(angle);
    dy = cos(angle);
    i  = 1;
    
    %% Main loop
    xx = startx;
    yy = starty;
    
    while image(yy,xx) == 1
        
        % result
        x = xx;
        y = yy;
        
        % next pixel in angle given direction
        xdda = startx + floor(i*dx);
        ydda = starty + floor(i*dy);
        
        % image boundaries overflow control
        xdda = min(xdda, size(image,2));
        xdda = max(xdda, 0);
        ydda = min(ydda, size(image,1));
        
        % new direction
        if image(ydda,xdda) == 1
            yy = ydda;
            xx = xdda;
        else
            yy = yy + 1; %down
            starty = starty+1;
        end
        
        i = i + 1;
    end
end